/**
The contents of this file are subject to the Mozilla Public License Version 1.1 
(the "License"); you may not use this file except in compliance with the License. 
You may obtain a copy of the License at http://www.mozilla.org/MPL/ 
Software distributed under the License is distributed on an "AS IS" basis, 
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the 
specific language governing rights and limitations under the License. 

The Original Code is "UnderlyingAccessor.java".  Description: 
"This class is used to determine the name of the accessor method on the 
underlying object for a given conformance class. It is used to determine 
whether underlying accessors require a "rep" parameter. For instance,
a message type might provide an accessor <code>getMSH()</code> to return
its MSH segment. The runtime profile may not provide any clue as to whether
the MSH segment in the HAPI library for the given message type is repeatable
or not. This class determines this information."     

The Initial Developer of the Original Code is University Health Network. Copyright (C) 
2001.  All Rights Reserved. 

Contributor(s): James Agnew
                Paul Brohman
                Mitch Delachevrotiere
                Shawn Dyck
  				    Cory Metcalf
  				
Alternatively, the contents of this file may be used under the terms of the 
GNU General Public License (the  ?GPL?), in which case the provisions of the GPL are 
applicable instead of those above.  If you wish to allow use of your version of this 
file only under the terms of the GPL and not to allow others to use your version 
of this file under the MPL, indicate your decision by deleting  the provisions above 
and replace  them with the notice and other provisions required by the GPL License.  
If you do not delete the provisions above, a recipient may use your version of 
this file under either the MPL or the GPL. 

*/
package ca.uhn.hl7v2.conf.classes.generator.builders;

import java.lang.reflect.*;
import ca.uhn.hl7v2.conf.classes.exceptions.*;
import ca.uhn.hl7v2.Log;

/** 
 * This class is used to determine the name of the accessor method on the 
 * underlying object for a given conformance class. It is used to determine 
 * whether underlying accessors require a "rep" parameter. For instance,
 * a message type might provide an accessor <code>getMSH()</code> to return
 * its MSH segment. The runtime profile may not provide any clue as to whether
 * the MSH segment in the HAPI library for the given message type is repeatable
 * or not. This class determines this information.     
 *  
 * @author <table><tr><td>James Agnew</td></tr>
 *                <tr><td>Paul Brohman</td></tr>
 *                <tr><td>Mitch Delachevrotiere</td></tr>
 *                <tr><td>Shawn Dyck</td></tr>
 * 				   <tr><td>Cory Metcalf</td></tr></table>
 */
public class UnderlyingAccessor {
   boolean acceptsRep;

   String theAccessor;

   /**
    * Determines the name of the accessor method on the underlying object for a given
    * conformance class. It is used to determine whether underlying accessors require
    * a "rep" parameter.   
    * @param className the Class name 
    * @param accessorName the Accessor ame
    */
   public UnderlyingAccessor(String className, String accessorName) {
      boolean foundAccessor = false;
      acceptsRep = false;

		Class c;
      try {
         c = Class.forName(className);
         Method methods[] = c.getMethods();
         for (int i = 0; i < methods.length; i++) {
            if (methods[i].getName().equals(accessorName)) {
               foundAccessor = true;
               if (methods[i].getParameterTypes().length == 1) {
                  acceptsRep = true;
                  break;
               }
            }

            // TODO: Remove this line when all testing is complete	
            //System.out.println( "\n\nDoes  " + methods[i].getName() + "\nequal " + accessorName ); 

         }

         if (foundAccessor == false)
				throw new ConformanceError("Underlying HAPI class \"" + className + "\" does not have accessor \"" + accessorName + "()\". This is probably a bug.");

      } catch (Exception e) {
         Log.tryToLog(e, "Underlying class/method " + className + "." + accessorName + "() can not be found. The complete HAPI API must be installed prior to using this system.");
         throw new ConformanceError("Underlying class/method " + className + "." + accessorName + "() can not be found. The complete HAPI API must be installed prior to using this system.");
      }

      theAccessor = new String(accessorName);
      theAccessor += acceptsRep ? "( rep )" : "()";
   }

   /**
    * Returns true if the underlying accessor accepts a rep argument, or
    * false if the underlying accessor does not.
    * @return boolean Returns true if the underlying accessor accepts a rep argument, or
    * false if the underlying accessor does not.
    */
   public boolean getAcceptsRep() {
      return acceptsRep;
   }

   /** This method returns a string representation of the Accessor
    * @return a String representation of the Accessor
    * @see java.lang.Object#toString()
    */
   public String toString() {
      return theAccessor;
   }

}
